@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2016 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2019 -- leonerd@leonerd.org.uk
 
 package Tickit;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 
 BEGIN {
-   our $VERSION = '0.61_004';
+   our $VERSION = '0.67_001';
 }
 
 use Carp;
@@ -24,14 +24,13 @@ BEGIN {
    XSLoader::load( __PACKAGE__, our $VERSION );
 }
 
+# We export some constants
+use Exporter 'import';
+
 use Tickit::Event;
+use Tickit::Term;
 use Tickit::Window;
 
-use Tickit::Debug;
-
-use Struct::Dumb;
-struct TimeQueue => [qw( time code )];
-
 =head1 NAME
 
 C<Tickit> - Terminal Interface Construction KIT
@@ -127,19 +126,17 @@ sub new
    my $class = shift;
    my %args = @_;
 
-   # Test code also accepts 'term' argument but we won't document that for now
-
-   my $in  = delete $args{term_in}  || \*STDIN;
-   my $out = delete $args{term_out} || \*STDOUT;
-
+   my $root = delete $args{root};
    my $term = delete $args{term};
 
    my $self = bless {
-      todo_queue => [],
-      timer_queue => [],
+      use_altscreen => $args{use_altscreen} // 1,
    }, $class;
 
-   unless( $term ) {
+   if( $args{term_in} or $args{term_out} ) {
+      my $in  = delete $args{term_in}  || \*STDIN;
+      my $out = delete $args{term_out} || \*STDOUT;
+
       my $writer = $self->_make_writer( $out );
 
       require Tickit::Term;
@@ -148,35 +145,13 @@ sub new
          writer        => $writer,
          input_handle  => $in,
          output_handle => $out,
-         UTF8          => $args{UTF8},
+         UTF8          => delete $args{UTF8},
       );
    }
 
    $self->{term} = $term;
-   $self->{term_in}  = $in;
-   $self->{term_out} = $out;
 
-   $self->{use_altscreen} = $args{use_altscreen} // 1;
-
-   my $rootwin = $self->{rootwin} = Tickit::Window->new( $self, $term->lines, $term->cols );
-
-   $self->bind_key( 'C-c' => $self->can( "stop" ) );
-
-   $term->set_output_buffer( 2**16 ); # 64KiB
-
-   weaken( my $weakself = $self );
-
-   $term->bind_event( key => sub {
-      $weakself or return;
-      my ( $term, $ev, $info ) = @_;
-      Tickit::Debug->log( Ik => "Key event %s %s (mod=%d)",
-         map { $info->$_ } qw( type str mod ) ) if DEBUG;
-      $weakself->on_key( $info );
-
-      return 1;
-   } );
-
-   $self->set_root_widget( $args{root} ) if defined $args{root};
+   $self->set_root_widget( $root ) if $root;
 
    return $self;
 }
@@ -195,6 +170,24 @@ sub _make_writer
    return $out;
 }
 
+sub _tickit
+{
+   my $self = shift;
+   return $self->{_tickit} //= do {
+      my $tickit = $self->_make_tickit( $self->{term} );
+
+      $tickit->setctl( 'use-altscreen' => $self->{use_altscreen} );
+
+      $tickit;
+   };
+}
+
+sub _make_tickit
+{
+   my $self = shift;
+   return Tickit::_Tickit->new( @_ );
+}
+
 =head2 later
 
    $tickit->later( $code )
@@ -205,27 +198,13 @@ events are processed.
 
 =cut
 
-sub _flush_later
-{
-   my $self = shift;
-
-   my $queue = $self->{todo_queue};
-   ( shift @$queue )->() while @$queue;
-}
-
-sub later
-{
-   my $self = shift;
-   my ( $code ) = @_;
-
-   push @{ $self->{todo_queue} }, $code;
-}
+sub later { shift->_tickit->watch_later( @_ ) }
 
 =head2 timer
 
- $tickit->timer( at => $epoch, $code )
+ $id = $tickit->timer( at => $epoch, $code )
 
- $tickit->timer( after => $delay, $code )
+ $id = $tickit->timer( after => $delay, $code )
 
 Runs the given CODE reference at some fixed point in time in the future.
 The first argmuent must be either the string C<at>, or C<after>; and specifies
@@ -233,6 +212,9 @@ that second argument gives either the absolute epoch time (C<$epoch>), or the
 delay relative to now (C<$delay>), respectively. Fractions are supported to a
 resolution of microseconds.
 
+Returns an opaque integer value that may be passed to L</cancel_timer>. This
+value is safe to ignore if not required.
+
 =cut
 
 sub timer
@@ -240,21 +222,30 @@ sub timer
    my $self = shift;
    my ( $mode, $amount, $code ) = @_;
 
-   my $at;
+   my $tickit = $self->_tickit;
+
    if( $mode eq "at" ) {
-      $at = $amount;
+      return $tickit->watch_timer_at_msec( $amount * 1000, $code );
    }
    elsif( $mode eq "after" ) {
-      $at = time + $amount;
+      return $tickit->watch_timer_after_msec( $amount * 1000, $code );
    }
    else {
       croak "Mode should be 'at' or 'after'";
    }
-
-   # TODO: bin-search insert position then splice
-   @{ $self->{timer_queue} } = sort { $a->time <=> $b->time } @{ $self->{timer_queue} }, TimeQueue( $at => $code );
 }
 
+=head2 cancel_timer
+
+ $tickit->cancel_timer( $id )
+
+Removes a timer previously installed by L</timer>. After doing so the code
+will no longer be invoked.
+
+=cut
+
+sub cancel_timer { shift->_tickit->watch_cancel( @_ ) }
+
 =head2 term
 
    $term = $tickit->term
@@ -263,11 +254,7 @@ Returns the underlying L<Tickit::Term> object.
 
 =cut
 
-sub term
-{
-   my $self = shift;
-   return $self->{term};
-}
+sub term { shift->_tickit->term }
 
 =head2 cols
 
@@ -285,25 +272,6 @@ of C<SIGWINCH> signals.
 sub lines { shift->term->lines }
 sub cols  { shift->term->cols  }
 
-sub _SIGWINCH
-{
-   my $self = shift;
-
-   $self->term->refresh_size;
-}
-
-sub on_key
-{
-   my $self = shift;
-   my ( $info ) = @_;
-
-   my $str = $info->str;
-
-   if( exists $self->{key_binds}{$str} ) {
-      $self->{key_binds}{$str}->( $self, $str ) and return;
-   }
-}
-
 =head2 bind_key
 
    $tickit->bind_key( $key, $code )
@@ -328,11 +296,38 @@ sub bind_key
    my $self = shift;
    my ( $key, $code ) = @_;
 
+   my $keybinds = $self->{key_binds} //= {};
+
    if( $code ) {
-      $self->{key_binds}{$key} = $code;
+      if( !%$keybinds ) {
+         weaken( my $weakself = $self );
+
+         # Need to ensure a root window exists before this so it gets its
+         # key bind event first
+         $self->rootwin;
+
+         $self->{key_bind_id} = $self->term->bind_event( key => sub {
+            my $self = $weakself or return;
+            my ( $term, $ev, $info ) = @_;
+            my $str = $info->str;
+
+            if( my $code = $self->{key_binds}{$str} ) {
+               $code->( $self, $str );
+            }
+
+            return 0;
+         } );
+      }
+
+      $keybinds->{$key} = $code;
    }
    else {
-      delete $self->{key_binds}{$key};
+      delete $keybinds->{$key};
+
+      if( !%$keybinds ) {
+         $self->term->unbind_event_id( $self->{key_bind_id} );
+         undef $self->{key_bind_id};
+      }
    }
 }
 
@@ -344,11 +339,8 @@ Returns the root L<Tickit::Window>.
 
 =cut
 
-sub rootwin
-{
-   my $self = shift;
-   return $self->{rootwin};
-}
+# root window needs to know where the toplevel "tickit" instance is
+sub rootwin { $_[0]->_tickit->rootwin( $_[0] ) }
 
 =head2 set_root_widget
 
@@ -365,65 +357,6 @@ sub set_root_widget
    ( $self->{root_widget} ) = @_;
 }
 
-=head2 setup_term
-
-   $tickit->setup_term
-
-Set up the screen and generally prepare to start running
-
-=cut
-
-sub setup_term
-{
-   my $self = shift;
-
-   my $term = $self->term;
-
-   $term->await_started( 0.100 ); # 100 msec
-
-   $term->setctl_int( altscreen => 1 ) if $self->{use_altscreen};
-   $term->setctl_int( cursorvis => 0 );
-   $term->setctl_int( mouse     => Tickit::Term::TERM_MOUSEMODE_DRAG );
-   $term->clear;
-
-   if( my $widget = $self->{root_widget} ) {
-      $widget->set_window( $self->rootwin );
-   }
-
-   $term->flush;
-}
-
-=head2 teardown_term
-
-   $tickit->teardown_term
-
-Shut down the screen after running
-
-=cut
-
-sub teardown_term
-{
-   my $self = shift;
-
-   if( my $widget = $self->{root_widget} ) {
-      $widget->set_window( undef );
-   }
-
-   my $term = $self->term;
-
-   if( $self->{use_altscreen} ) {
-      $term->setctl_int( altscreen => 0 );
-   }
-   else {
-      $term->goto( $term->get_size );
-   }
-
-   $term->setctl_int( cursorvis => 1 );
-   $term->setctl_int( mouse     => 0 );
-
-   $term->flush;
-}
-
 =head2 tick
 
    $tickit->tick
@@ -433,50 +366,13 @@ C<teardown_term>.
 
 =cut
 
-sub _flush
-{
-   my $self = shift;
-
-   $self->_flush_later if @{ $self->{todo_queue} };
-   $self->rootwin->flush if $self->rootwin;
-}
-
-sub _tick
-{
-   my $self = shift;
-
-   my $timer_queue = $self->{timer_queue};
-
-   my $timeout;
-   if( @$timer_queue ) {
-      $timeout = $self->{timer_queue}[0]->time - time;
-   }
-
-   $self->{term}->input_wait( $timeout );
-
-   my $now = time;
-   while( @$timer_queue and $timer_queue->[0]->time <= $now ) {
-      shift( @$timer_queue )->code->();
-   }
-
-   $self->_flush;
-}
-
 sub tick
 {
    my $self = shift;
 
-   my $old_DIE = $SIG{__DIE__};
-   local $SIG{__DIE__} = sub {
-      local $SIG{__DIE__} = $old_DIE;
+   # TODO: Consider root widget
 
-      die @_ if $^S;
-
-      $self->teardown_term;
-      die @_;
-   };
-
-   $self->_tick;
+   $self->_tickit->tick( @_ );
 }
 
 =head2 run
@@ -493,40 +389,17 @@ sub run
 {
    my $self = shift;
 
-   $self->setup_term;
+   if( my $widget = $self->{root_widget} ) {
+      $widget->set_window( $self->rootwin );
+   }
 
-   $SIG{INT} = $SIG{TERM} = sub {
-      my $signal = shift;
+   # TODO: $SIG{__DIE__} wrapping
 
-      # Disable the handler so a second signal will be fatal
-      undef $SIG{$signal};
-      $SIG{$signal} = sub {
-         die "Aborting Tickit on double SIGINT\n";
-      };
+   $self->_tickit->run;
 
-      $self->stop;
-   };
-
-   $SIG{WINCH} = sub {
-      $self->later( sub { $self->_SIGWINCH } )
-   };
-
-   my $old_DIE = $SIG{__DIE__};
-   local $SIG{__DIE__} = sub {
-      local $SIG{__DIE__} = $old_DIE;
-
-      die @_ if $^S;
-
-      $self->teardown_term;
-      die @_;
-   };
-
-   $self->_flush;
-
-   local $self->{keep_running} = 1;
-   $self->_tick while( $self->{keep_running} );
-
-   $self->teardown_term;
+   if( my $widget = $self->{root_widget} ) {
+      $widget->set_window( undef );
+   }
 }
 
 =head2 stop
@@ -537,11 +410,7 @@ Causes a currently-running C<run> method to stop processing events and return.
 
 =cut
 
-sub stop
-{
-   my $self = shift;
-   $self->{keep_running} = 0;
-}
+sub stop { shift->_tickit->stop( @_ ) }
 
 =head1 AUTHOR
 
