@@ -8,7 +8,7 @@ package Tickit::Event;
 use strict;
 use warnings;
 
-our $VERSION = '0.61_004';
+our $VERSION = '0.67_001';
 
 use Carp;
 
@@ -23,12 +23,6 @@ are invoked they receive an object instance to contain information about the
 event. Details of the event can be accessed as via accessor methods on these
 instances.
 
-For back-compatibility, each event type that C<Tickit::Term> can emit also
-provides a hash reference overload method that yields a hash containing keys
-named after the accessors. This code will be removed in a later version as per
-deprecation policy. Existing widget code should be changed to use the new
-object methods rather than the now-legacy hash keys.
-
 =head1 ACCESSORS
 
 The following methods are shared between C<Tickit::Event::Key> and
@@ -45,9 +39,9 @@ modifiers is set.
 
 =cut
 
-sub mod_is_alt   { shift->mod & Tickit::Term::MOD_ALT }
-sub mod_is_ctrl  { shift->mod & Tickit::Term::MOD_CTRL }
-sub mod_is_shift { shift->mod & Tickit::Term::MOD_SHIFT }
+sub mod_is_alt   { shift->mod & Tickit::MOD_ALT }
+sub mod_is_ctrl  { shift->mod & Tickit::MOD_CTRL }
+sub mod_is_shift { shift->mod & Tickit::MOD_SHIFT }
 
 package
    Tickit::Event::Expose;
@@ -95,12 +89,6 @@ package
    Tickit::Event::Key;
 our @ISA = qw( Tickit::Event );
 
-use overload '%{}' => sub {
-   my $self = shift;
-   warnings::warnif( deprecated => "HASH access to Tickit::Event::Key is deprecated" );
-   return +{ map { $_ => $self->$_ } qw( type str mod ) };
-};
-
 =head1 Tickit::Event::Key
 
 =head2 type
@@ -121,12 +109,6 @@ package
    Tickit::Event::Mouse;
 our @ISA = qw( Tickit::Event );
 
-use overload '%{}' => sub {
-   my $self = shift;
-   warnings::warnif( deprecated => "HASH access to Tickit::Event::Mouse is deprecated" );
-   return +{ map { $_ => $self->$_ } qw( type button line col mod ) };
-};
-
 =head1 Tickit::Event::Mouse
 
 =head2 type
@@ -154,12 +136,6 @@ package
    Tickit::Event::Resize;
 our @ISA = qw( Tickit::Event );
 
-use overload '%{}' => sub {
-   my $self = shift;
-   warnings::warnif( deprecated => "HASH access to Tickit::Event::Resize is deprecated" );
-   return +{ map { $_ => $self->$_ } qw( lines cols ) };
-};
-
 =head1 Tickit::Event::Resize
 
 =head2 lines
