@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2013-2016 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2013-2017 -- leonerd@leonerd.org.uk
 
 package Tickit::RenderBuffer;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.61_004';
+our $VERSION = '0.67_001';
 
 use Carp;
 use Scalar::Util qw( refaddr );
@@ -335,7 +335,7 @@ Successive calls to this method will replace the active pen used, but if there
 is a saved state on the stack it will be merged with the rendering pen of the
 most recent saved state.
 
-This method may be preferrable to passing pens into multiple C<text> or
+This method may be preferable to passing pens into multiple C<text> or
 C<erase> calls as it may be more efficient than merging the same pen on every
 call. If the original pen is still required afterwards, the C<savepen> /
 C<restore> pair may be useful.
@@ -374,6 +374,14 @@ and no buffer changes are made.
 
 =cut
 
+=head2 skiprect
+
+   $rb->skiprect( $rect )
+
+Sets the range of cells given by the rectangle to skipped state.
+
+=cut
+
 =head2 text_at
 
    $cols = $rb->text_at( $line, $col, $text, $pen )
@@ -501,7 +509,7 @@ the segment should go to the centre of each cell, or should continue through
 the entire body of the cell and stop at the boundary. By default line segments
 will start and end at the centre of the cells, so that horizontal and vertical
 lines meeting in a cell will form a neat corner. When drawing isolated lines
-such as horizontal or vertical rules, it is preferrable that the line go right
+such as horizontal or vertical rules, it is preferable that the line go right
 through the cells at the start and end. To control this behaviour, the
 C<$caps> bitmask is used. C<CAP_START> and C<CAP_END> state that the line
 should consume the whole of the start or end cell, respectively; C<CAP_BOTH>
@@ -579,6 +587,23 @@ efficient than many single-column wide C<text_at> calls.
 
 =cut
 
+=head2 copyrect
+
+=head2 moverect
+
+   $rb->copyrect( $dest, $src )
+
+   $rb->moverect( $dest, $src )
+
+Copies (or moves) buffered content from one rectangular region to another.
+The two regions may overlap.
+
+The move operation is identical to the copy operation followed by setting the
+vacated areas of the source rectangle not covered by the destination to
+skipping state.
+
+=cut
+
 =head2 get_cell
 
    $cell = $rb->get_cell( $line, $col )
