@@ -1,19 +1,19 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2016 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2018 -- leonerd@leonerd.org.uk
 
 package Tickit::Term;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.61_004';
+our $VERSION = '0.67_001';
 
 use Carp;
 
 # Load the XS code
-use Tickit;
+use Tickit qw( MOD_SHIFT MOD_ALT MOD_CTRL BIND_FIRST );
 
 # We export some constants
 use Exporter 'import';
@@ -27,6 +27,8 @@ use constant {
 
 push our @EXPORT_OK, qw(
    TERM_CURSORSHAPE_BLOCK TERM_CURSORSHAPE_UNDER TERM_CURSORSHAPE_LEFT_BAR
+   MOD_SHIFT MOD_ALT MOD_CTRL
+   BIND_FIRST
 );
 
 =head1 NAME
@@ -111,6 +113,15 @@ sub new
    return $self;
 }
 
+=head2 open_stdio
+
+   $term = Tickit::Term->open_stdio
+
+Convenient shortcut for obtaining a L<Tickit::Term> instance bound to the
+STDIN and STDOUT streams of the process.
+
+=cut
+
 =head1 METHODS
 
 =cut
@@ -172,6 +183,33 @@ seconds.
 
 =cut
 
+=head2 pause
+
+   $term->pause
+
+Suspends operation of the terminal by resetting it to its default state.
+
+=cut
+
+=head2 resume
+
+   $term->resume
+
+Resumes operation of the terminal after a L</pause>.
+
+Typically these two methods are used together, either side of a blocking wait
+around a C<SIGSTOP>.
+
+   sub suspend
+   {
+      $term->pause;
+      kill STOP => $$;
+      $term->resume;
+      $rootwin->expose;
+   }
+
+=cut
+
 =head2 flush
 
    $term->flush
@@ -200,7 +238,7 @@ L<Tickit::Event>.
 
    $id = $term->bind_event( $ev, $flags, $code, $data )
 
-The C<$code> argument may optionally be preceeded by an integer of flag
+The C<$code> argument may optionally be preceded by an integer of flag
 values. This should be zero to apply default semantics, or a bitmask of one or
 more of the following constants:
 
@@ -229,19 +267,6 @@ sub bind_event
    $self->_bind_event( $ev, $flags, $code, $data );
 }
 
-sub bind_event_with_flags
-{
-   croak "\term->bind_event_with_flags is deprecated; use ->bind_event";
-}
-
-sub bind_event_default
-{
-   my $self = shift;
-   my ( $ev, $code, $data ) = @_;
-   warnings::warnif( deprecated => "\term->bind_event_default is deprecated; use ->bind_event" );
-   $self->_bind_event( $ev, 0, $code, $data );
-}
-
 =head2 refresh_size
 
    $term->refresh_size
@@ -444,6 +469,17 @@ Sets the terminal window icon text, title, or both.
 
 =back
 
+=head2 getctl
+
+=head2 setctl
+
+   $value = $term->getctl( $ctl )
+
+   $success = $term->setctl( $ctl, $value )
+
+A newer form of the various typed get and set methods above. This version
+will interpret the given value as appropriate, depending on the control type.
+
 =cut
 
 =head2 input_push_bytes
