@@ -8,7 +8,7 @@ package Tickit::Test;
 use strict;
 use warnings;
 
-our $VERSION = '0.61_004';
+our $VERSION = '0.67_001';
 
 use Carp;
 
@@ -141,7 +141,7 @@ sub mk_window
 
    my $win = $tickit->rootwin;
 
-   $tickit->setup_term;
+   #$tickit->setup_term;
 
    # Clear the method log from ->setup_term
    $term->get_methodlog;
