@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2016 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2019 -- leonerd@leonerd.org.uk
 
 package Tickit::Window;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use 5.010; # //
 
-our $VERSION = '0.61_004';
+our $VERSION = '0.67_001';
 
 use Carp;
 
@@ -208,7 +208,7 @@ L<Tickit::Event>.
 
    $id = $win->bind_event( $ev, $flags, $code, $data )
 
-The C<$code> argument may optionally be preceeded by an integer of flag
+The C<$code> argument may optionally be preceded by an integer of flag
 values. This should be zero to apply default semantics, or a bitmask of
 constants. The constants are documented in
 L<Tickit::Term/bind_event (with flags)>.
@@ -263,10 +263,6 @@ siblings.
 
 =cut
 
-sub _reap_dead_children
-{
-}
-
 =head2 parent
 
    $parentwin = $win->parent
@@ -300,10 +296,9 @@ Returns the root window
 Returns the L<Tickit::Term> instance of the terminal on which this window
 lives.
 
-For forward-compatibility with a C-library implementation of C<Tickit::Window>
-it is no longer guaranteed that this method will return the same Perl-level
-terminal instance that the root window was constructed with. In particular, if
-the root window in fact lives on a mock terminal created by
+Note that it is not guaranteed that this method will return the same
+Perl-level terminal instance that the root window was constructed with. In
+particular, if the root window in fact lives on a mock terminal created by
 L<Tickit::Test::MockTerm> this method may "forget" this fact, returning an
 object instance simply in the C<Tickit::Term> class instead. While the
 instance will still be useable as a terminal, the fact it was a mock terminal
@@ -377,95 +372,6 @@ than twice as would be the case calling the above methods individually.
 
 =cut
 
-sub set_on_geom_changed
-{
-   croak "\$win->set_on_geom_changed is deprecated; use ->bind_event";
-}
-
-sub set_on_key
-{
-   croak "\$win->set_on_key is deprecated; use ->bind_event";
-}
-
-sub _handle_key
-{
-   my $self = shift;
-   my ( $info ) = @_;
-
-   return 0 unless $self->is_visible;
-
-   $self->_reap_dead_children;
-   my $children = $self->{child_windows};
-   if( $children and @$children and $children->[0]->{steal_input} ) {
-      $children->[0]->_handle_key( $info ) and return 1;
-   }
-
-   my $focused_child = $self->{focused_child};
-
-   if( $focused_child ) {
-      $focused_child->_handle_key( $info ) and return 1;
-   }
-
-   $self->_fire_event( key => $info ) and return 1;
-
-   if( $children ) {
-      foreach my $child ( @$children ) {
-         next unless $child; # weakrefs; may be undef
-         next if $focused_child and $child == $focused_child;
-
-         $child->_handle_key( $info ) and return 1;
-      }
-   }
-
-   return 0;
-}
-
-sub set_on_mouse
-{
-   croak "\$win->set_on_mouse is deprecated; use ->bind_event";
-}
-
-sub _handle_mouse
-{
-   my $self = shift;
-   my ( $info ) = @_;
-
-   return unless $self->is_visible;
-
-   my $line = $info->line;
-   my $col  = $info->col;
-
-   if( my $children = $self->{child_windows} ) {
-      foreach my $child ( @$children ) {
-         next unless $child; # weakrefs; may be undef
-
-         my $child_line = $line - $child->top;
-         my $child_col  = $col  - $child->left;
-
-         if( !$child->{steal_input} ) {
-            next if $child_line < 0 or $child_line >= $child->lines;
-            next if $child_col  < 0 or $child_col  >= $child->cols;
-         }
-
-         my $child_info = Tickit::Event::Mouse->_new(
-            $info->type, $info->button, $info->line - $child->top, $info->col - $child->left, $info->mod,
-         );
-
-         my $ret = $child->_handle_mouse( $child_info );
-         return $ret if $ret;
-      }
-   }
-
-   $self->_fire_event( mouse => $info ) and return $self;
-
-   return 0;
-}
-
-sub set_on_expose
-{
-   croak "\$win->set_on_expose is deprecated; use ->bind_event";
-}
-
 our $INDENT = "";
 sub _do_expose
 {
@@ -520,10 +426,39 @@ window.
 
 =cut
 
-sub set_on_focus
-{
-   croak "\$win->set_on_focus is deprecated; use ->bind_event";
-}
+=head2 getctl
+
+=head2 setctl
+
+   $value = $win->getctl( $ctl )
+
+   $success = $win->setctl( $ctl, $value )
+
+Accessor and mutator for window control options. C<$ctl> should be one of the
+following options:
+
+=over 4
+
+=item cursor-blink (bool)
+
+=item cursor-shape (int)
+
+=item cursor-visible (bool)
+
+Cursor properties to set for the terminal cursor when this window has input
+focus.
+
+=item focus-child-notify (bool)
+
+Whether the window will also receive focus events about child windows.
+
+=item steal-input (bool)
+
+Whether the window is currently stealing input from its siblings.
+
+=back
+
+=cut
 
 =head2 set_focus_child_notify
 
@@ -536,6 +471,14 @@ receives notifications about the window itself.
 
 =cut
 
+sub set_focus_child_notify
+{
+   my $self = shift;
+   my ( $notify ) = @_;
+
+   $self->setctl( 'focus-child-notify' => $notify );
+}
+
 =head2 top
 
 =head2 bottom
@@ -844,6 +787,14 @@ when the window is focused.
 
 =cut
 
+sub set_cursor_visible
+{
+   my $self = shift;
+   my ( $visible ) = @_;
+
+   $self->setctl( 'cursor-visible' => $visible );
+}
+
 *cursor_visible = \&set_cursor_visible;
 
 =head2 cursor_shape
@@ -857,6 +808,14 @@ C<CURSORSHAPE_*> constants from L<Tickit::Term>.
 
 =cut
 
+sub set_cursor_shape
+{
+   my $self = shift;
+   my ( $shape ) = @_;
+
+   $self->setctl( 'cursor-shape' => $shape );
+}
+
 *cursor_shape = \&set_cursor_shape;
 
 =head2 take_focus
@@ -884,50 +843,6 @@ sub focus
    $self->take_focus;
 }
 
-sub _focus_gained
-{
-   my $self = shift;
-   my ( $child ) = @_;
-
-   if( $self->{focused_child} and defined $child and $self->{focused_child} != $child ) {
-      $self->{focused_child}->_focus_lost;
-   }
-
-   if( my $parent = $self->parent ) {
-      # Still update without ourself but don't inform parent if we're invisible
-      $parent->_focus_gained( $self ) if $self->is_visible;
-   }
-   else {
-      $self->_needs_restore;
-   }
-
-   if( !$child ) {
-      $self->{focused} = 1;
-      $self->_fire_event( focus => Tickit::Event::Focus->_new( in => undef ) );
-   }
-   elsif( $self->{focus_child_notify} ) {
-      $self->_fire_event( focus => Tickit::Event::Focus->_new( in => $child ) );
-   }
-
-   $self->{focused_child} = $child;
-   weaken $self->{focused_child} if WEAKEN_CHILDREN;
-}
-
-sub _focus_lost
-{
-   my $self = shift;
-
-   if( my $focused_child = $self->{focused_child} ) {
-      $focused_child->_focus_lost;
-      $self->_fire_event( focus => Tickit::Event::Focus->_new( out => $focused_child ) );
-   }
-
-   if( $self->{focused} ) {
-      undef $self->{focused};
-      $self->_fire_event( focus => Tickit::Event::Focus->_new( out => undef ) );
-   }
-}
-
 =head2 is_focused
 
    $focused = $win->is_focused
@@ -936,74 +851,35 @@ Returns true if this window currently has the input focus
 
 =cut
 
-=head2 restore
+=head2 is_steal_input
 
-   $win->restore
+   $steal = $win->is_steal_input
 
-Restore the state of the terminal to its idle state. Places the cursor back
-at the focus position, and restores the pen.
+Returns true if this window is currently stealing input from its siblings
 
 =cut
 
-sub restore
-{
-   my $self = shift;
-   my $root = $self->root;
-
-   my $term = $root->term;
-
-   my $win = $root;
-   while( $win ) {
-      last unless $win->is_visible;
-      last unless $win->{focused_child};
-      $win = $win->{focused_child};
-   }
-
-   if( $win and $win->is_focused and $win->{cursor_visible} and
-       ( $win->_get_span_visibility( $win->{cursor_line}, $win->{cursor_col} ) )[0] ) {
-      my $cursorshape = $win->{cursor_shape} // Tickit::Term::CURSORSHAPE_BLOCK;
-      $term->setctl_int( cursorvis => 1 );
-      $win->term->goto( $win->{cursor_line} + $win->abs_top, $win->{cursor_col} + $win->abs_left );
-      $win->term->setctl_int( cursorshape => $cursorshape );
-   }
-   else {
-      $term->setctl_int( cursorvis => 0 );
-   }
-
-   $term->flush;
-}
-
-sub clearline
-{
-   croak "\$win->clearline has been removed";
-}
-
-sub clear
+sub is_steal_input
 {
    my $self = shift;
 
-   croak "\$win->clear on non-root window has been removed" if $self->parent;
-
-   carp "\$win->clear on root window is deprecated and will be removed";
-
-   my $term = $self->term;
-   $term->setpen( $self->get_effective_pen );
-   $term->clear;
-
-   $self->_needs_flush;
+   return $self->getctl( 'steal-input' );
 }
 
-sub _needs_flush
+=head2 set_steal_input
+
+   $win->set_steal_input( $steal )
+
+Controls whether this window is currently stealing input from its siblings
+
+=cut
+
+sub set_steal_input
 {
    my $self = shift;
-   return if $self->{flush_queued};
+   my ( $steal ) = @_;
 
-   $self->tickit->later( sub {
-      $self->term->flush;
-      undef $self->{flush_queued};
-   } );
-
-   $self->{flush_queued}++;
+   $self->setctl( 'steal-input' => $steal );
 }
 
 sub sprintf
@@ -1129,14 +1005,14 @@ the containing window.
 
 Emitted when the window gains or loses input focus.
 
-If the C<focus_child_notify> behavior is enabled, this callback is also
+If the C<focus-child-notify> behavior is enabled, this callback is also
 invoked for changes of focus on descendent windows. In this case, it is
 passed an additional argument, being the immediate child window in which the
 focus chain has now changed (which may or may not be the focused window
 directly; it could itself be another ancestor).
 
 When a window gains focus, any of its ancestors that have
-C<focus_child_notify> enabled will be informed first, from the outermost
+C<focus-child-notify> enabled will be informed first, from the outermost
 inwards, before the window itself. When one loses focus, it is notified
 first, and then its parents from the innermost outwards.
 
